package main

import (
	"fmt"
	"math/rand"
	"time"
)

// Live Coding Timeline
// --------------------
// The following is a potential timeline for a 30 Minute coding session,
// implementing the Block Drop Game task.
//
// T+0  create dummy main.go
// T+5  added Game struct and Block struct as first proposed data structures
// T+10 added addBlock, Run, and Show method stubs for showing how to manage the game state
// T+15 added Drop and Step method stub and added some first case in the game loop
// T+20 added some logic in Step function, changed color from string to Color(int)
// T+25 added ClearLines function, score variable, and "game over" return
// T+30 "finished!" and feeling sad about completing so few of the described features 😿

type Game struct {
	// grid [16][8]string // T+5 initally drafted colors as string
	grid [16][8]Color // T+20 changed to Color(int)

	currentBlock *Block
}

type Block struct {
	x int
	y int

	// color string // T+5 initally drafted colors as string
	color Color // T+20 changed to Color(int)
}

type Color int

func (g *Game) addBlock() {
	// add new block on top
	// T+10 Question: in the middle or a random place? -> Let's try random!

	// T+20: implement random block addition
	g.currentBlock = &Block{
		x:     rand.Int() % 8,
		y:     -1,                    // 0 is the top, 16 is the bottom the blocks starts above row 0
		color: Color(rand.Int() % 5), // RGB + BW
	}
}

func (g *Game) Run() {
	// start the game
	// we need some notion of time passing by or a notion of game steps
	// this will need some loop that advances the game by
	// * checking the game conditions and rules
	// * applying changes on user input
	// * recompute the new positions of falling blocks

	fmt.Println("starting game loop")

	tick := time.NewTicker(time.Second)
	steps := 0
	score := 0
	defer tick.Stop()

	for {
		select {
		case <-tick.C:
			steps += 1
			fmt.Println("advance game state to step =", steps)
			if g.Step() != nil {
				// T+25 add game over case
				fmt.Println("game over score = ", score)
				return
			}
			score += g.ClearLines() // T+25 add ClearLine and score
		}
	}
}

func (g *Game) Step() error {
	if g.currentBlock == nil {
		g.addBlock()
	}
	if err := g.Drop(g.currentBlock); err != nil {
		// cannot drop the block any further
		// * check if block is stuck and return gameover error
		// * else "move" the currentBlock to the grid and end the step
		g.currentBlock = nil
	}
	return nil
}

func (g *Game) Drop(block *Block) error {
	// try to move the given block one step downwards.
	return nil
}

func (g *Game) ClearLines() int {
	// * check grid for lines and sum up score
	return 0
}

func (g *Game) Show() {
	for _, v := range g.grid {
		fmt.Println(v)
	}
}

func main() {
	fmt.Println("starting block game")

	g := Game{}
	fmt.Println("start grid:")
	g.Show()
	g.Run()
}
